#include <inttypes.h>
#include "bsp.h"
#include "led.h"
#include "spi.h"
#include "tty.h"
#include "eth-config.h"
#include "enc28j60.h"
#include "ether.h"
#include "uip/uip.h"
#include "uip/uip_arp.h"
#include "uip/clock.h"
#include "uip/timer.h"
#include "flash.h"

#define BUF ((struct uip_eth_hdr *)&uip_buf[0])

static struct timer periodic_timer, arp_timer;

void
ether_init_uip ()
{
  unsigned char mac[6];
  uip_ipaddr_t ipaddr;

  timer_set(&periodic_timer, CLOCK_SECOND);
  timer_set(&arp_timer, CLOCK_SECOND * 10);

  spi_ethernet ();
  enc28j60Init ();
  uip_init();
  
  uip_ipaddr(ipaddr,
	     flash_data.ether.ip_addr[0],
	     flash_data.ether.ip_addr[1],
	     flash_data.ether.ip_addr[2],
	     flash_data.ether.ip_addr[3]);
  uip_sethostaddr(ipaddr);

  uip_ipaddr(ipaddr,
	     flash_data.ether.gw_addr[0],
	     flash_data.ether.gw_addr[1],
	     flash_data.ether.gw_addr[2],
	     flash_data.ether.gw_addr[3]);
  uip_setdraddr(ipaddr);

  uip_ipaddr(ipaddr,
	     flash_data.ether.netmask[0],
	     flash_data.ether.netmask[1],
	     flash_data.ether.netmask[2],
	     flash_data.ether.netmask[3]);
  uip_setnetmask(ipaddr);

  uip_setethaddr ((*(struct uip_eth_addr *)flash_data.ether.mac_addr));
  nicSetMacAddress (flash_data.ether.mac_addr);
  nicGetMacAddress (mac);
  cprintf("mac is %x\n", mac, 6);
}

void
ether_poll_uip ()
{
  int i;
  uip_len = pmenc_read();
  if(uip_len > 0) {
    if(BUF->type == htons(UIP_ETHTYPE_IP)) {
      uip_arp_ipin();
      uip_input();
      /* If the above function invocation resulted in data that
	 should be sent out on the network, the global variable
	 uip_len is set to a value > 0. */
      if(uip_len > 0) {
	uip_arp_out();
	pmenc_send();
      }
    } else if(BUF->type == htons(UIP_ETHTYPE_ARP)) {
      uip_arp_arpin();
      /* If the above function invocation resulted in data that
	 should be sent out on the network, the global variable
	 uip_len is set to a value > 0. */
      if(uip_len > 0) {
	pmenc_send();
      }
    }

  } else if(timer_expired(&periodic_timer)) {
    timer_reset(&periodic_timer);
    for(i = 0; i < UIP_CONNS; i++) {
      uip_periodic(i);
      /* If the above function invocation resulted in data that
	 should be sent out on the network, the global variable
	 uip_len is set to a value > 0. */
      if(uip_len > 0) {
	uip_arp_out();
	pmenc_send();
      }
    }

#if UIP_UDP
    for(i = 0; i < UIP_UDP_CONNS; i++) {
      uip_udp_periodic(i);
      /* If the above function invocation resulted in data that
	 should be sent out on the network, the global variable
	 uip_len is set to a value > 0. */
      if(uip_len > 0) {
	uip_arp_out();
	pmenc_send();
      }
    }
#endif /* UIP_UDP */
      
    /* Call the ARP timer function every 10 seconds. */
    if(timer_expired(&arp_timer)) {
      timer_reset(&arp_timer);
      uip_arp_timer();
    }
  }
}

void
ether_diag (int short_test)
{
  int i, v;
  if (short_test)
    {
      v = enc28j60Read(EREVID);
      cprintf("ENC28J60 Revision %d\n", v);
      return;
    }
}
