#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/uio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "../packet.h"
#include "../htonl.h"

#define MAX_MESG_SIZE sizeof(PacketStruct)
unsigned char mesg[MAX_MESG_SIZE];
PacketStruct *packet = (PacketStruct *)mesg;

long long saved_energy[4][16];
int first_time[4];
long sums[4];
double card_kwhm[4];
double card_kwh[4];

void
gotorc(int row, int col)
{
  printf("\033[%d;%dH", row, col+1);
}

double
kwh_to_dollars (double kwh)
{
  return kwh * 0.1596;
}
#define MONTHLY_FEE 20.0

char *comma(long long l)
{
  char buf[21];
  static char buf2[30];
  int i, j, sign;
  if (l < 0)
    {
      sign = -1;
      l = -l;
    }
  else
    sign = 1;
  sprintf(buf, "%020lld", l);
  for (j=1, i=0; i<20; i++)
    {
      if (i % 3 == 2)
	buf2[j++] = ',';
      buf2[j++] = buf[i];
    }
  buf2[j] = 0;
  i = 1;
  while ((buf2[i] == ',' || buf2[i] == '0') && buf2[i+1])
    i++;
  if (sign < 0)
    {
      i--;
      buf2[i] = '-';
    }
  return buf2+i;
}

int
main()
{
  int udp_socket;
  struct sockaddr_in serverName;
  int size;
  int i;
  int card, crow;
  double kwh, secper;
  double prorate;
  int old_uptime[4];
  int uptime;
  int ud, uh, um, us;

  udp_socket = socket (PF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (udp_socket == -1)
    {
      perror("socket");
      exit(1);
    }

  memset(&serverName, 0, sizeof(serverName));

  serverName.sin_family = AF_INET;
  serverName.sin_addr.s_addr = htonl(INADDR_ANY);
  serverName.sin_port = htons(PM_PORT);

  if (bind(udp_socket, (struct sockaddr *)&serverName, sizeof(serverName)) < 0)
    {
      perror("bind()");
      exit(1);
    }

  memset (first_time, 1, sizeof(first_time));
  for (i=0; i<4; i++)
    card_kwh[i] = card_kwhm[i] = 0.0;

  printf("\033[1;1H\033[2J");

  while (1)
    {
      size = recv(udp_socket, mesg, MAX_MESG_SIZE, 0);
      if (size == -1)
        {
	  fflush(stdout);
	  perror("recvfrom()");
	  exit(1);
        }
      if (memcmp (packet->magic, UDP_PACKET_MAGIC, sizeof(packet->magic)))
	continue;

      card = (htonl(packet->id) - 1);
      if (card < 0 || card > 3)
	continue;

#define WIDTH 40

      gotorc(1, card*WIDTH + 7);
      uptime = htonl(packet->uptime);
      us = uptime;
      um = us / 60; us %= 60;
      uh = um / 60; um %= 60;
      ud = uh / 24; uh %= 24;
      printf("up: %d+%d:%02d:%02d  id: %d\n", ud, uh, um, us, htonl(packet->id));

      crow = 0;
      sums[card] = 0;
      card_kwh[card] = 0.0;

      if (uptime)
	prorate = (30.0*24*60*60) / uptime;
      else
	prorate = 1;

      for (i=0; i<16; i++)
	{
	  long long e = htonll(packet->aenergy[i]);
	  long de;
	  int c = 0;
	  double cost;

	  e *= 0.966;

#if 0
	  if (e < 0)
	    e = 0;
#endif

	  crow ++;
	  if (i % 4 == 0) crow ++;
	  //if (i % 8 == 0) crow ++;

	  if (first_time[card] || uptime < 4 || uptime == old_uptime[card])
	    {
	      de = 0;
	    }
	  else
	    de = (e - saved_energy[card][i]) / (uptime - old_uptime[card]);

	  if (de < 0)
	    c = 226;
	  /* Greens: 231 194 157 120 83 46 */
	  else if (de < 3)
	    c = 243;
	  else if (de < 10)
	    c = de*2 + 237;
	  else if (de < 15)
	    c = 231;
	  else if (de < 30)
	    c = 194;
	  else if (de < 60)
	    c = 157;
	  else if (de < 120)
	    c = 120;
	  else if (de < 240)
	    c = 83;
	  else
	    c = 46;

	  /* "e" is watt seconds, we need to scale way down to get kilowatt hours.  */
	  kwh = e / (60.0 * 60.0 * 1000.0);
	  card_kwh[card] += kwh;

	  sums[card] += de;


	  gotorc(crow, card*WIDTH);
	  printf("\033[34m%2d\033[0m ", i);
	  //printf("J:\033[38;5;%dm%9.2f\033[0m kWh ", c, kwh * prorate);
	  printf("J:\033[38;5;%dm%12s\033[0m  ", c, comma(e));
	  printf("W:\033[38;5;%dm%5ld\033[0m ", c, de);

	  cost = kwh_to_dollars (kwh * prorate);
	  if (cost < 1)
	    c = cost * 15 + 240;
	  /* Cyans: 231 195 159 123 87 51 */
	  else if (cost < 4)
	    c = 231;
	  else if (cost < 8)
	    c = 195;
	  else if (cost < 16)
	    c = 159;
	  else if (cost < 32)
	    c = 123;
	  else if (cost < 64)
	    c = 87;
	  else
	    c = 51;

	  // Uncomment this to see amps, not dollars.
	  //cost = de / 120.0;

	  if (i == 15)
	    printf("\033[4m");
	  printf("\033[38;5;%dm$%6.2f\033[0m \n", c, cost);

	  saved_energy[card][i] = e;
	}
      first_time[card] = 0;
      old_uptime[card] = uptime;

      card_kwhm[card] = card_kwh[card] * prorate;

      gotorc (crow+1, card*WIDTH+22);
      printf("%4ld \033[38;5;157m$%6.2f \033[0m", sums[card], kwh_to_dollars (card_kwhm[card]));

#if 0
      gotorc (crow+3, 1*WIDTH+27);
      printf("\033[36m$%6.2f \033[0m", kwh_to_dollars (card_kwhm[0]+card_kwhm[1]));
#endif

      gotorc (crow+3, 1);
      i = sums[0] + sums[1] + sums[2] + sums[3];
      if (i > 0)
	secper = 3600 * 7.2 / i;
      else
	secper = 0;
      printf("current power: %4d watts (%.1f amps) %5.1f sec/rot ", i, i/240.0, secper);

      printf("  ($%.2f inst) ", i / 1000.0 * 24.0 * 30.0 * 0.16);

      gotorc (crow+4, 1);
      kwh = card_kwhm[0] + card_kwhm[1] + card_kwhm[2] + card_kwhm[3];
      printf("monthly usage: %.1f kWh \033[48;5;22m $%.2f \033[0m   ",
	     kwh, kwh_to_dollars (kwh) + MONTHLY_FEE);
#if 1
      kwh = card_kwh[0] + card_kwh[1] + card_kwh[2] + card_kwh[3];
      printf("  meter: %d  ", (int)kwh + 10660);
#endif

      fflush(stdout);
    }

    /* never reached */
    return 0;
}
