#include "bsp.h"
#include "flash.h"
#include "power.h"
#include "spi.h"
#include "tty.h"
#include "eth-config.h"

#define b(x) (unsigned char)~(1<<x)
#define z 0xff

unsigned char p0cs[] = { b(0), b(1), b(2), b(3), b(4), b(5), b(6), b(7),
			 z, z, z, z, z, z, z, z };
unsigned char p2cs[] = { z, z, z, z, z, z, z, z,
			 b(0), b(1), b(2), b(3), b(4), b(5), b(6), b(7) };

#undef b
#undef z

static const char * const reg_names[] = {
  "00",
  "WAVEFORM  ",
  "AENERGY   ",
  "RAENERGY  ",
  "LAENERGY  ",
  "VAENERGY  ",
  "RVAENERGY ",
  "LVAENERGY ",
  "LVARENERGY",
  "MODE      ",
  "IRQEN     ",
  "STATUS    ",
  "RSTSTATUS ",
  "CH1OS     ",
  "CH2OS     ",
  "GAIN      ",
  "PHCAL     ",
  "APOS      ",
  "WGAIN     ",
  "WDIV      ",
  "CFNUM     ",
  "CFDEN     ",
  "IRMS      ",
  "VRMS      ",
  "IRMSOS    ",
  "VRMSOS    ",
  "VAGAIN    ",
  "VADIV     ",
  "LINCYC    ",
  "ZXTOUT    ",
  "SAGCYC    ",
  "SAGLVL    ",
  "IPKLVL    ",
  "VPKLVL    ",
  "IPEAK     ",
  "RSTIPEAK  ",
  "VPEAK     ",
  "RSTVPEAK  ",
  "TEMP      ",
  "PERIOD    ",

  "28", "29", "2a", "2b", "2c", "2d", "2e", "2f", "30", "31", "32",
  "33", "34", "35", "36", "37", "38", "39", "3a", "3b", "3c",

  "TMODE     ",
  "CHKSUM    ",
  "DIEREV    "
};

static char reg_name_check[sizeof(reg_names)/sizeof(reg_names[0]) == 0x40 ? 1 : -1];

static char regsize[] = {
   0, 24, 24, 24, 24, 24, 24, 24, /* 00 - 07 */
  24, 16, 16, 16, 16,  8,  8,  8, /* 08 - 0f */
   6, 16, 12,  8, 12, 12, 24, 24, /* 10 - 17 */
  12, 12, 12,  8, 16, 12,  8,  8, /* 18 - 1f */
   8,  8, 24, 24, 24, 24,  8, 16, /* 20 - 27 */
   0,  0,  0,  0,  0,  0,  0,  0, /* 28 - 2f */
   0,  0,  0,  0,  0,  0,  0,  0, /* 30 - 37 */
   0,  0,  0,  0,  0,  8,  6,  8  /* 38 - 3f */
};
static char regsigned[] = {
  0, 1, 1, 1,  1, 0, 0, 0,   1, 0, 0, 0,  0, 0, 0, 0,  /* 00 - 0f */
  1, 1, 1, 0,  0, 0, 0, 0,   1, 1, 1, 0,  0, 0, 0, 0,  /* 10 - 1f */
  0, 0, 0, 0,  0, 0, 1, 0,   0, 0, 0, 0,  0, 0, 0, 0,  /* 20 - 2f */
  0, 0, 0, 0,  0, 0, 0, 0,   0, 0, 0, 0,  0, 0, 0, 0,  /* 30 - 3f */
};

static volatile long dump;

void
power_init_channel (int c, DF_Power *p)
{
      power_putreg (c, GAIN, flash_data.power[c].gain);
      power_putreg (c, MODE, flash_data.power[c].mode);
      power_putreg (c, LINCYC, flash_data.power[c].linecyc);
      power_putreg (c, IRMSOS, flash_data.power[c].irmsos);
      power_putreg (c, VRMSOS, flash_data.power[c].vrmsos);
      power_putreg (c, CH1OS, flash_data.power[c].ch1os);
      power_putreg (c, CH2OS, flash_data.power[c].ch2os);
      power_putreg (c, APOS, flash_data.power[c].apos);
      power_putreg (c, VAGAIN, flash_data.power[c].vagain);
      power_putreg (c, WGAIN, flash_data.power[c].wgain);
      power_putreg (c, PHCAL, flash_data.power[c].phcal);
      power_putreg (c, VADIV, flash_data.power[c].vadiv);
      power_putreg (c, WDIV, flash_data.power[c].wdiv);
}

void
power_init ()
{
  int c, rev, i;

  spi_power ();

  for (c=0; c<16; c++)
    power_init_channel (c, flash_data.power + c);

  wait_ms (20);

  for (i=0; i<0x40; i++)
    if (regsize[i])
      {
	wait_ms(3);
	for (c=0; c<16; c++)
	  dump = power_getreg (c, i);
      }
}

static void
delay_4us()
{
  int i;
  for (i=4*20; i; i--)
    asm volatile ("nop");
}

void
power_putreg (int channel, int reg, long val)
{
  int nbytes, i;

  reg &= 0x3f;

  nbytes = (regsize[reg] + 7) / 8;

  ENC_CS_IO = 1;
  p0.b = p0cs[channel];
  p2.b = p2cs[channel];
  delay_4us();
  spi_write (reg | 0x80);
  for (i=nbytes-1; i>=0; i--)
    spi_write (val >> (i * 8));
  delay_4us();
  p0.b = ~0;
  p2.b = ~0;
}

long
power_getreg (int channel, int reg)
{
  long rv = 0;
  int nbytes, i;

  reg &= 0x3f;

  nbytes = (regsize[reg] + 7) / 8;

  ENC_CS_IO = 1;
  p0.b = p0cs[channel];
  p2.b = p2cs[channel];
  delay_4us();
  spi_write (reg);
  delay_4us();
  for (i=nbytes-1; i>=0; i--)
    rv |= ((unsigned long)spi_read ()) << (i*8);
  delay_4us();
  spi_read ();
  p0.b = ~0;
  p2.b = ~0;
  if (regsigned[reg])
    switch (regsize[reg]) {
    case 24:
      rv <<= 8;
      rv >>= 8;
      break;
    case 16:
      rv = (signed short)rv;
      break;
    case 12:
      rv <<= 12;
      rv >>= 12;
      break;
    case 8:
      rv = (signed char)rv;
      break;
    default:
      break;
    }
  return rv;
}

void
power_diag (int channel, int short_test)
{
  int i;
  long v, apos;

  spi_power ();

  if (short_test)
    {
      v = power_getreg (channel, DIEREV);
      cprintf("Power[%d] Revision %d (0x%b)\n", channel, (int)v, (char)v);
      return;
    }

  for (i=0; i<0x40; i++)
    if (regsize[i])
      {
	v = power_getreg (channel, i);
	cprintf("Power[%d] %s\t%l\t%x\n", channel, reg_names[i], v, &v, (regsize[i]+7)/8);
      }

  apos = power_getreg (channel, APOS);
  wait_ms (1);
  for (i=0; i<17; i++)
    {
      long tp = 1L << i;
      power_putreg (channel, APOS, tp);
      wait_ms (1);
      v = power_getreg (channel, APOS);
      wait_ms (1);
      if ((int)v != (int)tp)
	cprintf("LINECYC r/w test: w %w r %w\n", (int)tp, (int)v);
    }
  power_putreg (channel, APOS, apos);
}
